%% CalcRK.m Simplified calculation of dynamic air cushion pressure 
% Also calculates force in lifting wire for parameter-study and
% natural periods of air cushion/water column
% Author: Torleif . Bertelsen 
% Last updated 10.03.2014
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clc
clear all
close all
%Defining characteristic dimensions and physical properties
D=3.4;                              %Diameter of suction anchor
H=5.955;                            %Height of suction anchor
t=0.02;                             %Wall thickness suction anchor
Ac=pi/4*(D)^2;                      %Area of air cushion
% dv=[0.61 1.02];                   %Diameter of small & large vent hatch
perf=0.019765;                      %Perforation ratio
dve=(D)*sqrt(perf);                 %Diameter of vent hatch
Al=(pi/4)*dve^2;                    %Leakage area
Patm=101300;                        %Atmospheric pressure
gamma_a=1.4;                        %Specific heat ratio of air
C_con=0.61;                         %Contraction coefficient
rho_a=1.225;                        %Density of air at 15 degC
rho_w=1025;                         %Density seawater
rho_s=7850;                         %Density steel
g=9.81;                             %Gravitational acceleration
%Wave data
Hs=0.000001;                        %Characteristic significant wave height
Tzmin=3.4;% Or (8.9)*sqrt(Hs/g);   %Min zero crossing T from DNV-RP-H103
                                   %Should be larger or equal to 3.4 [s]
Tzmax=13;                          %Upper zero crossing T from DNV-RP-H103
omega_max=(2*pi)/Tzmin;            %Maximum wave frequency from DNV-RP-H103
lambda=(g*Tzmin^2)/(2*pi);         %Wave length for the most critical Tz
k=2*pi/lambda;                     %Wave number for lowest wavelengths
count=1;                           %Plot counter
%Denoting time and u as symbols so they will not be calculated until 
%it is needed. (Here u is the dynamic pressure \mu) 
syms t_t u
%Operational data
w_const=-0.98;                      %Constant lowering velocity
w_var=-0.00000000001;                            %Varying lowering velocity
w_tot=w_const+w_var*t_t;            %Total lowering velocity
N_t=1000;                           %Number of steps
step=H/N_t;                         %Step size
h_sub=0:step:1.1*H;                 %Depth until total submergence + 10%
% time_test=h_subabs(w_tot);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%% Constant lowering velocity %%%%%%%%%%%%%%%%%%%%%%%%%%
%Set w_var=0

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%% Varying lowering velocity %%%%%%%%%%%%%%%%%%%%%%%
%Calculate the time during submergence with 2nd degree formula if the
%velocity is not constant
time_tot=zeros(1,length(h_sub));    %Define time during submergence
for i=1:length(h_sub)
    if w_var ~= 0
    time_tot(i)=(w_const+sqrt((w_const)^2+4*abs(w_var)*h_sub(i)))...
        /(2*abs(w_var));
    end
end
%Since the velocity is defined with a symbolic variable, the velocity must
%be evaluated with matlabFunction such that the numerical values can be
%stored in w_saved for later use.
F_wtot=matlabFunction(w_tot);
w_saved=F_wtot(time_tot);
d_sub=w_const*t_t+0.5*w_var*t_t^2;  %Integrated vel=Submergence level
Fsub=matlabFunction(d_sub);         %Saving the submergence level
dsub_saved=Fsub(time_tot);
%Wave elevation of considered incident waves evaluated at center of anchor
x_0=0;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%% Regular or constant waves %%%%%%%%%%%%%%%%%%%%%%%%%%
% Define incident wave elevation for regular waves
% zeta_inc=Hs/2;%*sin(k*x_0-omega_max*t_t);                %Max for sin()=1
% %Time derivative of the incident wave elevation
% zeta_inc_dot=omega_max*Hs/2;%*cos(k*x_0-omega_max*t_t);  %Max for cos()=1

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%% Irregular waves %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Call function which calculates surface elevation from JONSWAP spectrum
[omega_js1,zeta_js]=jonswap(time_tot);
%Use a random phase angle such that no dominating wave component is present
% epsilon=rand(1,length(time_tot))*2*pi;   
% epsilon=load('epsilonnew.mat');             %Random number vector [0,2pi]
% epsilon=new.epsilon;
% k_wave=omega_js1.^2/g;                     %Wave number deep water
%Calculating surface elevation by superimposing the elevations from the
%spectrum
% zeta_inc=sum(zeta_js.*cos(k_wave.*x_0-omega_js1.*t_t+epsilon)); 
% zeta_inc_dot=sum(zeta_js.*omega_js1.*sin(k_wave.*x_0...
% -omega_js1.*t_t+epsilon));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% Calculate ODE using Runge-Kutta 4th order method %%%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Non-linear function of \mu and t
F_tu=matlabFunction((zeta_inc_dot-w_tot-C_con*(Al/Ac)*...
    sqrt((2*u)/(rho_a)))/(1/(gamma_a*Patm)*(1+u/Patm)^(-1)*...
    (H+d_sub+u/(rho_w*g)-zeta_inc)+(1/(rho_w*g))));

uu = zeros(1,length(time_tot));                          %Initial condition
        
%Runge Kutta-4 Time integration loop  
% tic;
for i=1:(length(time_tot)-1)                             %Calculation loop
    a_1 = F_tu(time_tot(i),uu(i));
    a_2 = F_tu(time_tot(i)+0.5*step,uu(i)+0.5*step*a_1);
    a_3 = F_tu(time_tot(i)+0.5*step,uu(i)+0.5*step*a_2);
    a_4 = F_tu(time_tot(i)+step,uu(i)+a_3*step);
    
    uu(i+1) = uu(i) + (1/6)*(a_1+2*a_2+2*a_3+a_4)*step;  %main eq
    
%     error(i)=abs(uu(i+1)-uu(i));
%     Convergence criterion
%     if abs(uu(i+1)-uu(i))<1*10^-3
%         disp('conv');
%     else
%         break;
%     end
end
% toc                                             %End of time measurement
% figure(1)
% plot((0:step:t_sub-step),error);
% title('Residual \mu(i+1)-\mu(i)');
% % Plotting the dynamic pressure variation:
figure(count)
plot(time_tot,(uu));%,'LineWidth',3); set(gca,'FontSize',15);
title(['Time integration of dynamic air cushion pressure \mu with RK4'...
    'at no. of steps=', num2str(N_t)]);
xlabel('time [s]');
ylabel('\mu(t) [Pa]');

%Calculating total load in lifting wire including slamming force
%Define quantities to be used for the conical top
H_cyl=5.158472222;                      %Height of cylinder volume
H_cone=H-H_cyl;                         %Height of cone volume
t_cone=H_cone/abs(w_tot);               %Time until submergence of cyl-area
beta=atan(6.4/11.2);%beta=atan(5/11.1); %Deadrise angle in radians
F_hook=4500*g;                          %Hook load
F_anchor=12900*g+F_hook;                %Load of anchor in air
F_wire_vc=zeros(1,length(time_tot));    %Total load in lifting wire Von C.
F_wire_wag=zeros(1,length(time_tot));   %Total load in lifting wire Wagner
zeta_i=zeros(1,length(time_tot));       %Distance from SWL to inner
                                        %water surface
wsurf_i=zeros(1,length(time_tot));      %Water surface level inside suction
                                        %anchor  
wave_inc=zeros(1,length(time_tot));     %Saved wave elevation
wave_inc_dot=zeros(1,length(time_tot)); %Saved time-der. of wave elevation
sub_depth=zeros(1,length(time_tot));    %Saved submergence level

for i=1:length(time_tot)
    %Saving the wave elevation and its time derivative for later use
%     Constant or regular wave height
    wave_inc(i)=Hs/2;%*sin(k*x_0-omega_max*time_tot(i));
    wave_inc_dot(i)=omega_max*Hs/2;%*cos(k*x_0-omega_max*time_tot(i));
%     Irregular wave height
%     wave_inc(i)=sum(zeta_js.*cos(k_wave.*x_0...
%         -omega_js1.*time_tot(i)+epsilon));
%     wave_inc_dot(i)=sum(zeta_js.*omega_js1.*sin(k_wave.*x_0...
%         -omega_js1.*time_tot(i)+epsilon));
    zeta_i(i)=(wave_inc(i)-uu(i)/(rho_w*g));    
    sub_depth(i)=w_saved(i)*time_tot(i);
    wsurf_i(i)=sub_depth(i)-(zeta_i(i));  %Defined as negative
    
    F_buoy_wag=uu(i)*Ac;          %Load for the cylinder area
    F_buoy_vc=uu(i)*Ac;           %Load for the cylinder area
    last=F_buoy_vc;               %Initial value for the load on cone area 
        
    if wsurf_i(i) <= -H_cyl  && wsurf_i(i) >= -1.0075*H  %Slamming criteria
               
        %Start-time(=0) for calculating load on cone area
        tstep_cone(i)=wsurf_i(i)/w_saved(i)+H_cyl/w_saved(i);
        
        %Load for the cone area with Von Karman's wetted length
        F_vc=last+rho_w*(pi*2)*(w_saved(i)^3/(tan(beta))^2)*...
    (-(tstep_cone(i)*(pi*D/4))+(w_saved(i)*(tstep_cone(i))^2)/(tan(beta)));
        %Include initial load from \mu at transition cylinder to cone area
            F_buoy_vc=F_vc;
        %Load for the cone area with Wagner's wetted length
        F_wag=last+(rho_w*pi^4/4)*(w_saved(i)^3/(tan(beta))^2)*...
     (-(tstep_cone(i)*(D/2))+(w_saved(i)*(tstep_cone(i))^2)/(4*tan(beta)));
            F_buoy_wag=F_wag;
    end
    F_wire_wag(i)=F_anchor-abs(F_buoy_wag);
    F_wire_vc(i)=F_anchor-abs(F_buoy_vc);
end
%Plotting realization of the incident waves generated from the JONSWAP
%spectrum
% figure(2)
% plot(time_tot,wave_inc,'LineWidth',3); set(gca,'FontSize',15);
% title('Realizations of regular waves');
% xlabel('time [s]');
% ylabel('Surface elevation');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Comparison-data from Subsea7's full scale measurement
d_fs=[0 1 2 3 4 5 5.5 5.8 6];
d_ss7=[0 1 2 3 4 5 6];
F_fs=[F_anchor 149112 142245 130473 127333.8... 
    123017.4 119878.2 86524.2 62195.4; F_anchor 147542.4 136947.6...
    128903.4 125175.6 119878.2 119289.6 77499 75733.2;...
    F_anchor 143030 138713.4 129688 126745 131650 143422 106144 94176];
F_mean=mean(F_fs,1);
% F_fit=fit(time_tot,F_mean','smoothingspline');
F_ss7=[F_anchor 147000 146000 132000 130000 131000 98000];
count=count+1;
figure(count)
plot(abs(sub_depth),F_wire_vc,d_fs,F_mean);%,'LineWidth',3);
title('Force in lifting wire versus submergence');
ylabel('Force [N]');
xlabel('Submergence [m]')
legend('Best-fit with Von Karman slamming','Force measured');
axis([0 6 0 2*10^5])
%set(gca,'FontSize',15);
% plot(abs(dsub_saved),F_wire_vc, abs(dsub_saved),...
% F_wire_wag,d_fs,F_fs(1,:), d_fs, F_fs(2,:),d_fs,F_fs(3,:)...
% ,'LineWidth',3);
% 
% legend('Decreased contraction coefficient',...
%      'Average of full-scale measurements');
% % Plotting the total force in the wire vs depth
% title(['Total force in crane wire during submergence']);% with'...
% %     ,num2str(eta3_dot),' [m/s] lowering speed',...
% %     ' and contraction coefficient =',num2str(C_con)]);
% axis([0 6 0 2*10^5])
% xlabel('Depth [m]');
% ylabel('Force in lifting wire [N]');
% grid on;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Calculate natural frequency for oscillating air cushion / water(piston)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Vol_c=zeros(1,length(time_tot));       %Instantaneous air cushion volume
A33_disk=9790;                         %3D-Added mass of disk
A33_water=51800;                       %3D-Added mass of fluid from WAMIT
uu_dot=zeros(1,length(time_tot));
omega_wn1=zeros(1,length(time_tot));
omega_wn2=zeros(1,length(time_tot));
T_wn1=zeros(1,length(time_tot));
T_wn2=zeros(1,length(time_tot));
% 
for i=1:length(time_tot)


    Vol_c(i)=Ac*(H-zeta_i(i)+w_saved(i)*time_tot(i));
    omega_wn1(i)=sqrt((g+(Ac*gamma_a*(Patm+uu(i)))/(rho_w*Vol_c(i)))/...
        ((4*D)/(3*pi)-wsurf_i(i)));
    T_wn1(i)=2*pi/omega_wn1(i);  %Natural period of water and air coupled

%Calculating Natural freq. for oscillating water and air: Miles' approach
    %Defining the natural frequency of the oscillating water with the 
    %dynamic air cushion pressure as the upper boundary (coupled)

    omega_wn2(i)=sqrt(g/((4*D)/(3*pi)-wsurf_i(i)));
    %Defining the natural frequency of the oscillating water with the 
    %atmospheric pressure as the upper boundary (uncoupled)
    T_wn2(i)=2*pi/omega_wn2(i);   %Natural period of water uncoupled
        if wsurf_i(i) <= -H_cyl*0.999 && wsurf_i(i) >= -H_cyl*1.0001
%             disp(['Air cushion volume is',num2str(Vol_c(i))]);

%Calculating Natural frequency for oscillating air cushion: Sloshing appr
            omega_cn=Ac*sqrt(gamma_a*(Patm+uu(i))/(A33_water*Vol_c(i)));
             %Natural period air cushion
            T_cn1=2*pi/omega_cn;

    %Defining the natural frequency with use of added mass instead of the
    %dynamic air cushion pressure
    omega_wn3=sqrt(g/((4*D)/(3*pi)-wsurf_i(i)-A33_water/(rho_w*Ac)));
        
%         T_wn1=2*pi/omega_wn1   %Natural period of water and air coupled 
%         T_wn2=2*pi/omega_wn2   %Natural period of water uncoupled
        T_wn3=2*pi/omega_wn3;   %Natural period of water with added mass
        
        end
        %Calculating frequency according to measured volume and pressure
%         if H+wsurf_i(i)>=0.99*0.174 && H+wsurf_i(i)<=1.01*0.174
%             wsurf_i(i)
%         vol_meas=pi/3*0.174*0.914^2;
%         P_meas=4.8939e+03;
%         omega0=sqrt((g+(Ac*gamma_a*(Patm+uu(i)))/(rho_w*vol_meas))/...
%         ((4*D)/(3*pi)-wsurf_i(i)));
%         T_0=2*pi/omega0        %Natural period corresp. to measured P,V
%         end
end
% count=count+1;
% figure(count)
% plot(abs(sub_depth),T_wn1,abs(sub_depth),T_wn2);
% legend('Air cushion and water coupled','Water column');
% title(['Natural period comparison vs depth']);
% xlabel('Depth [s]');
% ylabel('Natural period T [s]');


